//Copyright (c) 2009 Gabriel See
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:

//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.

//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.

//ALPHA LegoRoboBrick
#include "MCLib/MCLib.c"
#include "BT.h"

//definitions. TR is top radius
#define TR 7
//To is top offset
#define To (6.0 + (sqrt(3.0)/2.0))
//Bo is bottom offset
#define Bo (2.0 + 4.0/sqrt(3.0))
//BR is bottom radius
#define BR 14
//Gear is Gear ratio
#define Gear 1
float ALPHAMath(float X, float Y, float Z)
{
	float R1,R2,D,Theta1,Theta2,Xo;
	int nme; //Number motor encoder
	R1=TR;
	R2=sqrt(BR*BR-Y*Y);
	Xo=To-Bo-X;
	D=sqrt(Xo*Xo+Z*Z);
	Theta1=180*(acos((D*D+R1*R1-R2*R2)/(2*R1*D)))/PI;
	Theta2=180*asin(Xo/D)/PI;
	nme=Gear*(Theta1-Theta2);
	return nme;
}
//p is for "position of "
void ALPHAABC(float X, float Y, float Z, int &pA, int &pB, int &pC)
{
	float XR, YR;
	pA=ALPHAMath(X,Y,Z);//calculate A motor
	XR = X*cosDegrees(120) - Y*sinDegrees(120);//Rotate B into A spot
	YR = X*sinDegrees(120) + Y*cosDegrees(120);
	pB=ALPHAMath(XR,YR,Z);//calculate B motor
	XR = X*cosDegrees(-120) - Y*sinDegrees(-120);//Rotate C into A spot
	YR = X*sinDegrees(-120) + Y*cosDegrees(-120);
	pC=ALPHAMath(XR,YR,Z);//calculate C motor
}
void ALPHAInit()
{
	while (bQueuedMsgAvailable()) {
    ClearMessage();   // Were ready to process the next message
  }
	MCInit(motorA, motorB, motorC);// Initialize motors
	MCSetSync(motorA, motorB, motorC);//Synchronize motors
	MCFindLimit(motorA, 25, 165);//Motor limit
	MCSetTarget(motorA,90);
	MCSetTarget(motorB,90);
	MCSetTarget(motorC,90);
	wait1Msec(100);
	MCStart();
	wait1Msec(500);
}
void ALPHAMove(float X, float Y, float Z)
{
	static float Xc, Yc, Zc;
	int A,B,C;
	float R,Xd,Yd,Zd,Xn,Yn,Zn;
	while((Xc!=X)||(Yc!=Y)||(Zc!=Z))
	{
		R=sqrt((X-Xc)*(X-Xc)+(Y-Yc)*(Y-Yc)+(Z-Zc)*(Z-Zc));
		if(R<1)
		{
			Xn=X;
			Yn=Y;
			Zn=Z;
		}else
		{
			Xd=(X-Xc)/R;
			Yd=(Y-Yc)/R;
			Zd=(Z-Zc)/R;
			Xn=Xd+Xc;
			Yn=Yd+Yc;
			Zn=Zd+Zc;
		}
		ALPHAABC(Xn,Yn,Zn,A,B,C);
		MCSetTarget(motorA,A);
		MCSetTarget(motorB,B);
		MCSetTarget(motorC,C);
		Xc=Xn;
		Yc=Yn;
		Zc=Zn;
		wait1Msec(100);
	}
}
/*****************************ALPHA Lib***********************************/
//Definitions
#define UP 8
#define TIP 0
#define DOWN (12-TIP)
#define IN (15-TIP)
#define INPD (16-TIP)
/*typedef struct
{
	float X=0;
	float Y=0;
} T_Petri_Dish;*/
//Subroutines
void PipetSuck(float X, float Y)
{
	ALPHAMove(X, Y, UP);
	wait1Msec(1000);
	ALPHAMove(X, Y, DOWN);
	wait1Msec(1000);
	ALPHAMove(X, Y, IN);
	wait1Msec(1000);
	sendMessageWithParm(BT_ASPIRATE, 10);
	BTwaitForDone();
	ALPHAMove(X, Y, DOWN);
	wait1Msec(1000);
	ALPHAMove(X, Y, UP);
	wait1Msec(1000);
}

void PipetDispense(float X, float Y)
{
	ALPHAMove(X, Y, DOWN);
	wait1Msec(1000);
	ALPHAMove(X, Y, IN);
	wait1Msec(1000);
	sendMessageWithParm(BT_DISPENSE);
	BTwaitForDone();
	ALPHAMove(X, Y, UP);
	wait1Msec(1000);
}

void PipetClean(float X, float Y)
{
	ALPHAMove(X, Y, UP);
	wait1Msec(1000);
	ALPHAMove(X, Y, DOWN);
	wait1Msec(1000);
	ALPHAMove(X, Y, INPD);
	wait1Msec(1000);
	//Insert cleaning motion here
	sendMessageWithParm(BT_CLEAN);
	wait1Msec(5000);
	BTwaitForDone();
	ALPHAMove(X, Y, DOWN);
	wait1Msec(1000);
	ALPHAMove(X, Y, UP);
	wait1Msec(1000);
}
